@@ -12,7 +12,7 @@ Getencoding <- function(url) {
 
   pag<-tryCatch(GET(url, user_agent("Mozilla/5.0 (Windows NT 6.3; WOW64; rv:42.0) Gecko/20100101 Firefox/42.0"),timeout(5)) , error=function(e) NULL)
   if (!is.null(pag)){
-      page<-as.character(content(pag, type = "htmlTreeParse", as="text"))
+      page<-as.character(content(pag, type = "htmlTreeParse", as="text", encoding = "UTF-8"))
       if (grepl("<meta http-equiv=\"Content-Type\"", page)) {
         enc<-xml_attr(xml_find_one(read_html(page),"//meta[@http-equiv=\'Content-Type\']"), "content")
         enc<-gsub(".*charset=([^\']*)","\\1",enc )
